UInt

Struct UInt 

pub struct UInt<const N: usize>(/* private fields */);
Generic Unsigned integer type composed of 64-bit digits, of arbitrary fixed size which must be known at compile time.

Digits are stored in little endian (the least significant digit first). This integer type aims to exactly replicate the behaviours of Rust’s built-in unsigned integer types: u8, u16, u32, u64, u128 and usize. The const generic parameter N is the number of 64-bit digits that are stored.

Implementations

Source

impl<const N: usize> UInt<N>

pub const fn bitand(self, rhs: Self) -> Self

Performs the & operation.

pub const fn bitor(self, rhs: Self) -> Self

Performs the | operation.

pub const fn bitxor(self, rhs: Self) -> Self

Performs the ^ operation.

pub const fn not(self) -> Self

Performs the unary ! operation.

Source

impl<const N: usize> UInt<N>

pub const fn carrying_mul(self, rhs: Self, carry: Self) -> (Self, Self)

Calculates the “full multiplication” self * rhs + carrywithout the possibility to overflow.

This returns the low-order (wrapping) bits and the high-order (overflow) bits of the result as two separate values, in that order.

Performs “long multiplication” which takes in an extra amount to add, and may return an additional amount of overflow. This allows for chaining together multiple multiplications to create “big integers” which represent larger values.

If you don’t need the carry, then you can use Self::widening_mul instead.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.carrying_mul.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = U256::MAX;
let b = U256::MAX;
let c = U256::ZERO;

assert_eq!(a.carrying_mul(b, c), (U256::ONE, b - U256::ONE));
Source

impl<const N: usize> UInt<N>

Checked arithmetic methods which act on self: self.checked_.... Each method cannot panic and returns an Option<Self>. None is returned when overflow would have occurred or there was an attempt to divide by zero or calculate a remainder with a divisor of zero.

pub const fn checked_add(self, rhs: Self) -> Option<Self>

pub const fn checked_sub(self, rhs: Self) -> Option<Self>

pub const fn checked_div(self, rhs: Self) -> Option<Self>

pub const fn checked_div_euclid(self, rhs: Self) -> Option<Self>

pub const fn checked_rem(self, rhs: Self) -> Option<Self>

pub const fn checked_rem_euclid(self, rhs: Self) -> Option<Self>

pub const fn checked_shl(self, rhs: u32) -> Option<Self>

pub const fn checked_shr(self, rhs: u32) -> Option<Self>

pub const fn checked_pow(self, pow: u32) -> Option<Self>

pub const fn checked_next_multiple_of(self, rhs: Self) -> Option<Self>

pub const fn checked_neg(self) -> Option<Self>

Source

impl<const N: usize> UInt<N>

pub const fn checked_add_signed(self, rhs: Int<N>) -> Option<Self>

pub const fn checked_mul(self, rhs: Self) -> Option<Self>

pub const fn checked_ilog2(self) -> Option<u32>

pub const fn checked_ilog10(self) -> Option<u32>

pub const fn checked_ilog(self, base: Self) -> Option<u32>

pub const fn checked_next_power_of_two(self) -> Option<Self>

Returns the smallest power of two greater than or equal to self.

If the next power of two is greater than Self::MAX, None is returned, otherwise the power of two is wrapped in Some.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.checked_next_power_of_two.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(2);
assert_eq!(n.checked_next_power_of_two(), Some(n));

let m = u256!(3);
assert_eq!(U256::MAX.checked_next_power_of_two(), None);
pub const fn checked_power_of_ten(power: u32) -> Option<Self>

Returns an integer whose value is 10^power.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::checked_power_of_ten(2), Some(u256!(100)));
pub const fn checked_power_of_five(power: u32) -> Option<Self>

Returns an integer whose value is 5^power.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::checked_power_of_five(2), Some(u256!(25)));
pub const fn checked_mul_digit(self, digit: u64) -> Option<Self>

Checked integer multiplication by single u64 digit. Computes self * rhs, returning None if overflow occurred.

pub const fn checked_add_digit(self, digit: u64) -> Option<Self>

Source

impl<const N: usize> UInt<N>

pub const fn eq(&self, other: &Self) -> bool

Tests for self and other values to be equal, and is used by ==.

pub const fn ne(&self, other: &Self) -> bool

Tests for self and other values to be not equal, and is used by !=.

pub const fn is_zero(&self) -> bool

Returns whether self is zero.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert!(U256::ZERO.is_zero());
assert!(!U256::ONE.is_zero());
pub const fn is_one(&self) -> bool

Returns whether self is one.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert!(U256::ONE.is_one());
assert!(!U256::MAX.is_one());
pub const fn cmp(&self, other: &Self) -> Ordering

This method returns an core::cmp::Ordering between self and other.

By convention, self.cmp(&other) returns the ordering matching the expression self <operator> other if true.

pub const fn max(self, other: Self) -> Self

Compares and returns the maximum of two values.

Returns the second argument if the comparison determines them to be equal.

pub const fn min(self, other: Self) -> Self

Compares and returns the minimum of two values.

Returns the first argument if the comparison determines them to be equal.

pub const fn clamp(self, min: Self, max: Self) -> Self

Restrict a value to a certain interval.

Returns max if self is greater than max, and min if self is less than min.Otherwise this returns self.# Panics

Panics if min > max.

pub const fn lt(&self, other: &Self) -> bool

Tests less than (for self and other) and is used by the < operator.

pub const fn le(&self, other: &Self) -> bool

Tests less than or equal to (for self and other) and is used by the <= operator.

pub const fn gt(&self, other: &Self) -> bool

Tests greater than (for self and other) and is used by the > operator.

pub const fn ge(&self, other: &Self) -> bool

Tests greater than or equal to (for self and other) and is used by the >= operator.

Source

impl<const N: usize> UInt<N>

pub const fn from_u64(n: u64) -> Self

Converts u64 to U256.

pub const fn parse_str(s: &str) -> Self

Parse U256 from string using hexadecimal, binary or decimal base.

Panics

This function will panic if U256 can’t be constructed from a given string.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::parse_str("0b1"), u256!(1));
assert_eq!(U256::parse_str("0xA"), u256!(10));
assert_eq!(U256::parse_str("12345"), u256!(12345));
pub const fn parse_str_radix(s: &str, radix: u32) -> Self

Parse U256 from string using a given base to an integer.

The string is expected to be an optional + sign followed by digits. Leading and trailing whitespace represent an error. Digits are a subset of these characters, depending on radix:

  • 0-9
  • a-z
  • A-Z
Panics

This function will panic if U256 can’t be constructed from a given string or if radix is not in the range from 2 to 36 inclusive.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::parse_str_radix("A", 16), u256!(10));
pub const fn from_str(s: &str) -> Result<Self, ParseError>

Try parse U256 from string using hexadecimal, binary or decimal base.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::from_str("0b1"), Ok(u256!(1)));
assert_eq!(U256::from_str("0xA"), Ok(u256!(10)));
assert_eq!(U256::from_str("12345"), Ok(u256!(12345)));
pub const fn from_str_radix(s: &str, radix: u32) -> Result<Self, ParseError>

Try parse U256 from string using a given base to an integer.

The string is expected to be an optional + sign followed by digits. Leading and trailing whitespace represent an error. Digits are a subset of these characters, depending on radix:

  • 0-9
  • a-z
  • A-Z
Panics

This function will panic if radix is not in the range from 2 to 36 inclusive.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::from_str_radix("A", 16), Ok(u256!(10)));
pub const fn parse_bytes(buf: &[u8], radix: u32) -> Option<Self>

Converts a byte slice in a given base to an U256 integer.

The input slice must contain ascii/utf8 characters in [0-9a-zA-Z].

This function is equivalent to the from_str_radix function for a string slice equivalent to the byte slice and the same radix.

Returns None if the conversion of the byte slice to string slice fails or if a digit is larger than or equal to the given radix, otherwise the integer is wrapped in Some.

Panics

This function will panic if radix is not in the range from 2 to 36 inclusive.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let src = "394857hdgfjhsnkg947dgfjkeita";
assert_eq!(U256::from_str_radix(src, 32).ok(), U256::parse_bytes(src.as_bytes(), 32));
pub const fn from_radix_be(buf: &[u8], radix: u32) -> Option<Self>

Converts a slice of big-endian digits in the given radix to an U256 integer.

Each u8 of the slice is interpreted as one digit of base radix of the number, so this function will return None if any digit is greater than or equal to radix, otherwise the integer is wrapped in Some.

Panics

This function will panic if radix is not in the range from 2 to 256 inclusive.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(3459874852685);
let digits = n.to_radix_be(12);
assert_eq!(Some(n), U256::from_radix_be(&digits, 12));
pub const fn from_radix_le(buf: &[u8], radix: u32) -> Option<Self>

Converts a slice of little-endian digits in the given radix to an U256 integer.

Each u8 of the slice is interpreted as one digit of base radix of the number, so this function will return None if any digit is greater than or equal to radix, otherwise the integer is wrapped in Some.

Panics

This function will panic if radix is not in the range from 2 to 256 inclusive.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(10983745987895);
let digits = n.to_radix_le(15);
assert_eq!(Some(n), U256::from_radix_le(&digits, 15));
pub const fn from_u8(n: u8) -> Self

Converts u8 to U256.

pub const fn from_u16(n: u16) -> Self

Converts u16 to U256.

pub const fn from_u32(n: u32) -> Self

Converts u32 to U256.

pub const fn from_usize(n: usize) -> Self

Converts usize to U256.

pub const fn from_u128(uint: u128) -> Result<Self, ParseError>

Converts u128 to U256.

pub const fn from_i8(int: i8) -> Result<Self, ParseError>

Converts i8 to U256.

pub const fn from_i16(int: i16) -> Result<Self, ParseError>

Converts i16 to U256.

pub const fn from_i32(int: i32) -> Result<Self, ParseError>

Converts i32 to U256.

pub const fn from_i64(int: i64) -> Result<Self, ParseError>

Converts i64 to U256.

pub const fn from_isize(int: isize) -> Result<Self, ParseError>

Converts isize to U256.

pub const fn from_i128(int: i128) -> Result<Self, ParseError>

Converts i128 to U256.

pub const fn from_f32(f: f32) -> Result<Self, ParseError>

Converts f32 to U256.

pub const fn from_f64(f: f64) -> Result<Self, ParseError>

Converts f64 to U256.

Source

impl<const N: usize> UInt<N>

pub fn to_str_radix(&self, radix: u32) -> String

Returns the U256 integer as a string in the given radix.

Panics

This function will panic if radix is not in the range from 2 to 36 inclusive.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let src = "934857djkfghhkdfgbf9345hdfkh";
let n = U256::from_str_radix(src, 36).unwrap();
assert_eq!(n.to_str_radix(36), src);
pub fn to_radix_be(&self, radix: u32) -> Vec<u8>

Returns the U256 integer in the given base in big-endian digit order.

Panics

This function will panic if radix is not in the range from 2 to 256 inclusive.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let digits = &[3, 55, 60, 100, 5, 0, 5, 88];
let n = U256::from_radix_be(digits, 120).unwrap();
assert_eq!(n.to_radix_be(120), digits);
pub fn to_radix_le(&self, radix: u32) -> Vec<u8>

Returns the U256 integer in the given base in little-endian digit order.

Panics

This function will panic if radix is not in the range from 2 to 256 inclusive.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let digits = &[1, 67, 88, 200, 55, 68, 87, 120, 178];
let n = U256::from_radix_le(digits, 250).unwrap();
assert_eq!(n.to_radix_le(250), digits);
pub const fn to_i8(self) -> Result<i8, ParseError>

Converts U256 to i8.

pub const fn to_i16(self) -> Result<i16, ParseError>

Converts U256 to i16.

pub const fn to_i32(self) -> Result<i32, ParseError>

Converts U256 to i32.

pub const fn to_i64(self) -> Result<i64, ParseError>

Converts U256 to i64.

pub const fn to_i128(self) -> Result<i128, ParseError>

Converts U256 to i128.

pub const fn to_isize(self) -> Result<isize, ParseError>

Converts U256 to isize.

pub const fn to_u8(self) -> Result<u8, ParseError>

Converts U256 to u8.

pub const fn to_u16(self) -> Result<u16, ParseError>

Converts U256 to u16.

pub const fn to_u32(self) -> Result<u32, ParseError>

Converts U256 to u32.

pub const fn to_u64(self) -> Result<u64, ParseError>

Converts U256 to u64.

pub const fn to_u128(self) -> Result<u128, ParseError>

Converts U256 to u128.

pub const fn to_usize(self) -> Result<usize, ParseError>

Converts U256 to usize.

Source

impl<const N: usize> UInt<N>

Methods which convert a UInt to and from data stored in different endianness.

pub const fn from_be(x: Self) -> Self

Converts an integer from big endian to the target’s endianness.

On big endian this is a no-op. On little endian the bytes are swapped.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.from_be.

pub const fn from_le(x: Self) -> Self

Converts an integer from little endian to the target’s endianness.

On little endian this is a no-op. On big endian the bytes are swapped.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.from_le.

pub const fn to_be(self) -> Self

Converts self from big endian to the target’s endianness.

On big endian this is a no-op. On little endian the bytes are swapped.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.to_be.

pub const fn to_le(self) -> Self

Converts self from little endian to the target’s endianness.

On little endian this is a no-op. On big endian the bytes are swapped.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.to_le.

pub const fn from_be_slice(slice: &[u8]) -> Option<Self>

Create an integer value from a slice of bytes in big endian.

The value is wrapped in an Option as the integer represented by the slice of bytes may represent an integer too large to be represented by the type.

If the length of the slice is shorter than Self::BYTES, the slice is padded with zeros or ones at the start so that it’s length equals Self::BYTES. It is padded with ones if the bytes represent a negative integer, otherwise it is padded with zeros.

If the length of the slice is longer than Self::BYTES, None will be returned, unless the bytes represent a non-negative integer and leading zeros from the slice can be removed until the length of the slice equals Self::BYTES, or if the bytes represent a negative integer and leading ones from the slice can be removed until the length of the slice equals Self::BYTES.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.from_be_slice.

pub const fn from_le_slice(slice: &[u8]) -> Option<Self>

Creates an integer value from a slice of bytes in little endian.

The value is wrapped in an Option as the bytes may represent an integer too large to be represented by the type.

If the length of the slice is shorter than Self::BYTES, the slice is padded with zeros or ones at the start so that it’s length equals Self::BYTES. It is padded with ones if the bytes represent a negative integer, otherwise it is padded with zeros.

If the length of the slice is longer than Self::BYTES, None will be returned, unless the bytes represent a non-negative integer and leading zeros from the slice can be removed until the length of the slice equals Self::BYTES, or if the bytes represent a negative integer and leading ones from the slice can be removed until the length of the slice equals Self::BYTES.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.from_le_slice.

Source

impl<const N: usize> UInt<N>

pub const fn count_ones(self) -> u32

Returns the number of ones in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.count_ones.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = u256!(7);

assert_eq!(a.count_ones(), 3);
pub const fn count_zeros(self) -> u32

Returns the number of zeros in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.count_zeros.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = U256::MAX;

assert_eq!(a.count_zeros(), 0);
pub const fn leading_zeros(self) -> u32

Returns the number of leading zeros in the binary representation of self.

Depending on what you’re doing with the value, you might also be interested in the Self::ilog2 function which returns a consistent number, even if the type widens.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.leading_zeros.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = U256::MAX;

assert_eq!(a.leading_zeros(), 0);
pub const fn trailing_zeros(self) -> u32

Returns the number of trailing zeros in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.trailing_zeros.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = u256!(4);

assert_eq!(a.trailing_zeros(), 2);
pub const fn leading_ones(self) -> u32

Returns the number of leading ones in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.leading_ones.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = U256::MAX;

assert_eq!(a.leading_ones(), 256);
pub const fn trailing_ones(self) -> u32

Returns the number of trailing ones in the binary representation of self.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.trailing_ones.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = u256!(3);

assert_eq!(a.trailing_ones(), 2);
pub const fn rotate_left(self, n: u32) -> Self

Shifts the bits to the left by a specified amount, n, wrapping the truncated bits to the end of the resulting integer.

Please note this isn’t the same operation as the << shifting operator!

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.rotate_left.

pub const fn rotate_right(self, n: u32) -> Self

Shifts the bits to the left by a specified amount, n, wrapping the truncated bits to the end of the resulting integer.

Please note this isn’t the same operation as the >> shifting operator! self.rotate_right(n) is equivalent to self.rotate_left(Self::BITS - n).

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.rotate_right.

pub const fn swap_bytes(self) -> Self

Reverses the byte order of the integer.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.swap_bytes.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(0x12345678901234567890123456789012);
assert_eq!(n.swap_bytes().swap_bytes(), n);
pub const fn reverse_bits(self) -> Self

Reverses the order of bits in the integer.

The least significant bit becomes the most significant bit, second least-significant bit becomes second most-significant bit, etc.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.reverse_bits.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(0x12345678901234567890123456789012);
assert_eq!(n.reverse_bits().reverse_bits(), n);
pub const fn pow(self, exp: u32) -> Self

Raises self to the power of exp, using exponentiation by squaring.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.pow.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(3);
assert_eq!(n.pow(5), u256!(243));
pub const fn add(self, rhs: Self) -> Self

Performs the + operation.

pub const fn shl(self, rhs: u32) -> Self

Performs the << operation.

pub const fn shr(self, rhs: u32) -> Self

Performs the >> operation.

pub const fn sub(self, rhs: Self) -> Self

Performs the - operation.

pub const fn rem(self, rhs: Self) -> Self

Performs the % operation.

pub const fn div_euclid(self, rhs: Self) -> Self

Performs Euclidean division.

Since, for the positive integers, all common definitions of division are equal, this is exactly equal to self / rhs.

Panics

This function will panic if rhs is zero.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.div_euclid.

pub const fn rem_euclid(self, rhs: Self) -> Self

Calculates the least remainder of self (mod rhs).

Since, for the positive integers, all common definitions of division are equal, this is exactly equal to self % rhs.

Panics

This function will panic if rhs is zero.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.rem_euclid.

pub const fn is_power_of_two(self) -> bool

Returns true if and only if self == 2^k for some integer k.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.is_power_of_two.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(8);
assert!(n.is_power_of_two());
let m = u256!(90);
assert!(!m.is_power_of_two());
pub const fn midpoint(self, rhs: Self) -> Self

Calculates the midpoint (average) between self and rhs.

midpoint(a, b) is (a + b) / 2 as if it were performed in a sufficiently-large unsigned integral type. This implies that the result is always rounded towards zero and that no overflow will ever occur.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.midpoint.

pub const fn ilog2(self) -> u32

Returns the base 2 logarithm of the number, rounded down.

Panics

This function will panic if self is zero.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.ilog2.

pub const fn ilog(self, base: Self) -> u32

Returns the logarithm of the number with respect to an arbitrary base, rounded down.

This method might not be optimized owing to implementation details; ilog2 can produce results more efficiently for base 2, and ilog10 can produce results more efficiently for base 10.# Panics

This function will panic if self is zero, or if base is less than 2.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.ilog.

pub const fn ilog10(self) -> u32

Find integer log10(x) of an integer.

fastnum use the most efficient algorithm based on relationship: log10(x) = log2(x)/log2(10)

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.ilog10.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(150);
assert_eq!(n.ilog10(), 2);
pub const fn next_multiple_of(self, rhs: Self) -> Self

Calculates the smallest value greater than or equal to self that is a multiple of rhs.

Panics

This function will panic if rhs is zero.

Overflow behavior

On overflow, this function will panic if overflow checks are enabled (default in debug mode) and wrap if overflow checks are disabled (default in release mode).

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.next_multiple_of.

pub const fn div_floor(self, rhs: Self) -> Self

Calculates the quotient of self and rhs, rounding the result towards negative infinity.

Panics

This function will panic if rhs is zero.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.div_floor.

pub const fn div_ceil(self, rhs: Self) -> Self

Calculates the quotient of self and rhs, rounding the result towards positive infinity.

Panics

This function will panic if rhs is zero.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.div_ceil.

pub const fn bits(&self) -> u32

Returns the smallest number of bits necessary to represent self.

This is equal to the size of the type in bits minus the leading zeros of self.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(u256!(0b1111001010100).bits(), 13);
assert_eq!(U256::ZERO.bits(), 0);
pub const fn bit(&self, b: u32) -> bool

Returns a boolean representing the bit in the given position (true if the bit is set).

The least significant bit is at index 0, the most significant bit is at index Self::BITS - 1.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(0b001010100101010101);
assert!(n.bit(0));
assert!(!n.bit(1));
assert!(!n.bit(U256::BITS - 1));
Source

impl<const N: usize> UInt<N>

pub const fn mul(self, rhs: Self) -> Self

Performs the * operation.

pub const fn div(self, rhs: Self) -> Self

Performs the / operation.

pub const fn neg(self) -> Self

Performs the unary - operation.

pub const fn digits(&self) -> &[u64; N]

Returns the digits stored in self as an array. Digits are little endian (least significant digit first).

pub fn digits_mut(&mut self) -> &mut [u64; N]

Returns the digits stored in self as a mutable array. Digits are little endian (least significant digit first).

pub const fn cast_signed(self) -> Int<N>

Returns the bit pattern of self reinterpreted as an unsigned integer of the same size.

This produces the same result as an as cast, but ensures that the bit-width remains the same.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.cast_signed.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = U256::MAX;

assert_eq!(a.cast_signed(), i256!(-1));
pub const fn div_digit(self, rhs: u64) -> Self

Performs division of a multi-precision integer by a single 64-bit digit.

This method implements an optimized division algorithm when the divisor is a single digit (u64). The optimization is significant because single-digit division is a common operation in decimal arithmetic and other numerical algorithms.

Algorithm
  1. For 64-bit numbers: Uses native CPU division
  2. For 128-bit numbers: Uses optimized two-word by one-word division
    1. For larger numbers: Uses long division algorithm with digit-by-digit processing
Returns

Returns the quotient as a new number of the same size as the dividend

Panics

This function will panic if digit is zero.

Performance

This operation is typically much faster than full multi-precision division, especially for larger numbers, as it avoids the complexity of multi-digit division.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(1000000000000000000);
assert_eq!(n.div_digit(1000000000), u256!(1000000000));
pub const fn div_rem(self, rhs: Self) -> (Self, Self)

Simultaneous truncated integer division and modulus.

Returns (quotient, remainder).

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(u256!(8).div_rem(u256!(3)), (u256!(2), u256!(2)));
pub const fn div_rem_digit(self, rhs: u64) -> (Self, u64)

Simultaneous truncated integer division and modulus.

Returns (quotient, remainder).

pub const fn mul_div_rem(self, rhs: Self, divisor: Self) -> (Self, Self)

Performs the self * rhs / divisor operation.

Returns (quotient, remainder).

pub const fn mul_div(self, rhs: Self, divisor: Self) -> Self

Performs the self * rhs / divisor operation.

Returns (quotient, remainder).

pub const fn from_digits(digits: [u64; N]) -> Self

Creates a new unsigned integer from the given array of digits. Digits are stored as little endian (least significant digit first).

pub const fn from_digit(digit: u64) -> Self

Creates a new unsigned integer from the given digit. The given digit is stored as the least significant digit.

pub const fn power_of_two(power: u32) -> Self

Returns an integer whose value is 2^power. This is faster than using a shift left on Self::ONE.

Panics

This function will panic if power is greater than or equal to Self::BITS.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = U256::power_of_two(11);
assert_eq!(n, (1u128 << 11).try_into().unwrap());
pub const fn power_of_five(power: u32) -> Self

Returns an integer whose value is 5^power.

Panics

This function will panic if 5^power is greater than Self::MAX

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::power_of_five(2), u256!(25));
pub const fn power_of_ten(power: u32) -> Self

Returns an integer whose value is 10^power.

Panics

This function will panic if 10^power is greater than Self::MAX

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::power_of_ten(2), u256!(100));
pub const fn abs_diff(self, other: Self) -> Self

Computes the absolute difference between self and other.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.abs_diff.

pub const fn mul_digit(self, digit: u64) -> Self

Integer multiplication by u64.

Computes self * digit, panicking if overflow occurred.

Overflow behavior

On overflow, this function will panic if overflow checks are enabled (default in debug mode) and wrap if overflow checks are disabled (default in release mode).

pub const fn decimal_digits(&self) -> u32

This method efficiently calculates the number of base-10 digits needed to represent the number without leading zeros.

The implementation uses optimized algorithms based on the size of the number.

Returns
  • Returns 0 for zero
  • For non-zero numbers, returns ⌊log10(n)⌋ + 1
Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(u256!(0).decimal_digits(), 0);
assert_eq!(u256!(1).decimal_digits(), 1);
assert_eq!(u256!(9).decimal_digits(), 1);
assert_eq!(u256!(10).decimal_digits(), 2);
assert_eq!(u256!(18446744073709551615).decimal_digits(), 20);
pub const fn remaining_decimal_digits(&self) -> u32

Calculates the maximum number of additional decimal digits that can be safely multiplied by this number without overflow.

This method is crucial for decimal arithmetic operations to prevent overflow when scaling numbers by powers of 10. The optimization is significant because single-digit division is a common operation in decimal arithmetic and other numerical algorithms.

Returns
  • For zero: Returns maximum allowed decimal digits
  • For non-zero numbers: Returns (MAX_POWER_10 + 1 - current_digits), adjusted if multiplication by 10^result would overflow
  • For zero: Returns maximum allowed decimal digits
Use Cases
  • Decimal scaling operations
  • Precision calculations
  • Overflow prevention in decimal arithmetic

This method is particularly useful in implementing decimal arithmetic where numbers need to be scaled while avoiding overflow conditions.

Performance

This operation is typically much slower than Self::can_scaled_by_power_of_ten,

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(u64!(18446744073709551615).remaining_decimal_digits(), 0); // cannot multiply by 10 (max U64 value is `18446744073709551615`)
assert_eq!(u64!(1844674407370955161).remaining_decimal_digits(), 1); // can multiply by 10 (it will be `18446744073709551610`)
assert_eq!(u64!(2844674407370955161).remaining_decimal_digits(), 0); // cannot multiply by 10 (one remaining decimal digit is formally exists but multiplication will overflow)
assert_eq!(u64!(24576).remaining_decimal_digits(), 14); // can multiply by 10^14
assert_eq!(u64!(14576).remaining_decimal_digits(), 15); // can multiply by 10^15

assert_eq!(u256!(115).remaining_decimal_digits(), 75); // can multiply by 10^75
assert_eq!(u256!(116).remaining_decimal_digits(), 74); // can multiply by 10^74 (max U256 value is `115792089237316195423570985008687907853269984665640564039457584007913129639935`)
pub const fn can_scaled_by_power_of_ten(&self, power: u32) -> bool

Checks if the number can be safely multiplied by a given power of 10 without overflow.

This method provides a fast way to check if scaling operations are safe without actually performing the multiplication.

Returns
  • true if the number can be multiplied by 10^power without overflow
  • false if such multiplication would overflow
Performance

Uses precomputed lookup table of maximum values divided by powers of 10 for efficient checking and avoid actual multiplication.

This operation is typically much faster than Self::remaining_decimal_digits,

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert!(!u64!(18446744073709551615).can_scaled_by_power_of_ten(1)); // cannot multiply by 10 (max U64 value is `18446744073709551615`)
assert!(u64!(1844674407370955161).can_scaled_by_power_of_ten(1)); // can multiply by 10 (it will be `18446744073709551610`)
assert!(!u64!(1844674407370955161).can_scaled_by_power_of_ten(2)); // can multiply by 10 (it will be `18446744073709551610`) but not 100
assert!(u64!(24576).can_scaled_by_power_of_ten(14)); // can multiply by 10^14
assert!(!u64!(24576).can_scaled_by_power_of_ten(15)); // can multiply by 10^14 not 10^15
assert!(u64!(14576).can_scaled_by_power_of_ten(15)); // can multiply by 10^15

Source

impl<const N: usize> UInt<N>

Overflowing arithmetic methods which act on self: self.overflowing_.... Each method returns a tuple of type (Self, bool) where the first item of the tuple is the result of the calculation truncated to the number of bits of self, and the second item is a boolean which indicates whether overflow occurred (i.e. if the number of bits of the result of the calculation exceeded the number of bits of self).

pub const fn overflowing_add(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_sub(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_div(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_div_euclid(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_rem(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_rem_euclid(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_shl(self, rhs: u32) -> (Self, bool)

pub const fn overflowing_shr(self, rhs: u32) -> (Self, bool)

pub const fn overflowing_pow(self, pow: u32) -> (Self, bool)

pub const fn overflowing_neg(self) -> (Self, bool)

Source

impl<const N: usize> UInt<N>

Overflowing arithmetic methods which act on self: self.overflowing_.... Each method returns a tuple of type (Self, bool) where the first item of the tuple is the result of the calculation truncated to the number of bits of self, and the second item is a boolean which indicates whether overflow occurred (i.e. if the number of bits of the result of the calculation exceeded the number of bits of self).

pub const fn overflowing_add_signed(self, rhs: Int<N>) -> (Self, bool)

pub const fn overflowing_add_digit(self, rhs: u64) -> (Self, bool)

pub const fn overflowing_mul(self, rhs: Self) -> (Self, bool)

pub const fn overflowing_mul_digit(self, rhs: u64) -> (Self, bool)

Calculates the multiplication of self and u64 rhs.

Returns a tuple of the multiplication along with a boolean indicating whether an arithmetic overflow would occur. If an overflow would have occurred then the wrapped value is returned.

Source

impl<const N: usize> UInt<N>

Saturating arithmetic methods which act on self: self.saturating_.... For each method, if overflow or underflow occurs, the largest or smallest value that can be represented by Self is returned instead.

Source

impl<const N: usize> UInt<N>

Saturating arithmetic methods which act on self: self.saturating_.... For each method, if overflow or underflow occurs, the largest or smallest value that can be represented by Self is returned instead.

Source

impl<const N: usize> UInt<N>

Strict arithmetic methods which act on self: self.strict_.... Each method will always panic if overflow/underflow occurs (i.e. when the checked equivalent would return None), regardless of whether overflow checks are enabled.

Source

impl<const N: usize> UInt<N>

pub const fn strict_add_signed(self, rhs: Int<N>) -> Self

pub const fn strict_power_of_ten(power: u32) -> Self

Returns an integer whose value is 10^power.

Panics

This function will panic if 10^power is greater than Self::MAX

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::strict_power_of_ten(2), u256!(100));
pub const fn strict_power_of_five(power: u32) -> Self

Returns an integer whose value is 5^power.

Panics

This function will panic if 5^power is greater than Self::MAX

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

assert_eq!(U256::strict_power_of_five(2), u256!(25));
pub const fn strict_mul(self, rhs: Self) -> Self

pub const fn strict_mul_digit(self, digit: u64) -> Self

Strict integer multiplication by u64.

Computes self * rhs, panicking if overflow occurred.

Panics

This function will always panic on overflow, regardless of whether overflow checks are enabled.

pub const fn strict_add_digit(self, digit: u64) -> Self

Source

impl<const N: usize> UInt<N>

This impl block contains no public items.

Widening arithmetic methods which act on self: self.widening_.... Each method returns of the calculation without the possibility to overflow.

Source

impl<const N: usize> UInt<N>

pub const fn widening_mul(self, rhs: Self) -> (Self, Self)

Calculates the complete product self * rhs without the possibility to overflow.

This returns the low-order (wrapping) bits and the high-order (overflow) bits of the result as two separate values, in that order.

If you also need to add a carry to the wide result, then you want Self::carrying_mul instead.

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.widening_mul.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let a = U256::MAX;
let b = U256::MAX;
assert_eq!(a.widening_mul(b), (U256::ONE, b - U256::ONE));
Source

impl<const N: usize> UInt<N>

Saturating arithmetic methods which act on self: self.saturating_.... For each method, if overflow or underflow occurs, the largest or smallest value that can be represented by Self is returned instead.

pub const fn wrapping_add(self, rhs: Self) -> Self

pub const fn wrapping_sub(self, rhs: Self) -> Self

pub const fn wrapping_mul(self, rhs: Self) -> Self

pub const fn wrapping_div(self, rhs: Self) -> Self

pub const fn wrapping_div_euclid(self, rhs: Self) -> Self

pub const fn wrapping_rem(self, rhs: Self) -> Self

pub const fn wrapping_rem_euclid(self, rhs: Self) -> Self

pub const fn wrapping_shl(self, rhs: u32) -> Self

pub const fn wrapping_shr(self, rhs: u32) -> Self

pub const fn wrapping_pow(self, pow: u32) -> Self

pub const fn wrapping_neg(self) -> Self

Source

impl<const N: usize> UInt<N>

Wrapping arithmetic methods which act on self: self.wrapping_.... Each method returns of the calculation truncated to the number of bits of self (i.e. they each return the first item in the tuple returned by their overflowing equivalent).

pub const fn wrapping_add_signed(self, rhs: Int<N>) -> Self

pub const fn wrapping_next_power_of_two(self) -> Self

Returns the smallest power of two greater than or equal to self.

If the next power of two is greater than Self::MAX, the return value is wrapped to Self::MIN

See also: https://doc.rust-lang.org/std/primitive.u64.html#method.wrapping_next_power_of_two.

Examples

Please note that this example is shared between integer types. Which explains why U256 is used here.

use fastnum::*;

let n = u256!(31);
assert_eq!(n.wrapping_next_power_of_two(), 32u32.into());
assert_eq!(U256::MAX.wrapping_next_power_of_two(), U256::MIN);
pub const fn wrapping_mul_digit(self, rhs: u64) -> Self

Wrapping (modular) multiplication by u64. Computes self * rhs, wrapping around at the boundary of the type.

Source

impl<const N: usize> UInt<N>

Associated constants for unsigned integer type.

pub const MIN: Self

The minimum value that this type can represent.

Examples

Please note that this example is shared between integer types. Which explains why U512 is used here.

use fastnum::*;

assert_eq!(!U512::MIN, U512::MAX);
pub const MAX: Self

The maximum value that this type can represent.

Examples

Please note that this example is shared between integer types. Which explains why U512 is used here.

use fastnum::*;

assert_eq!(U512::MAX.wrapping_add(U512::ONE), U512::MIN);
pub const BITS: u32 = BUint<N>::BITS

The total number of bits that this type contains.

Examples

Please note that this example is shared between integer types. Which explains why U512 is used here.

use fastnum::*;

assert_eq!(U512::BITS, 512);
pub const BYTES: u32 = BUint<N>::BYTES

The total number of bytes that this type contains.

Examples

Please note that this example is shared between integer types. Which explains why U512 is used here.

use fastnum::*;

assert_eq!(U512::BYTES, 512 / 8);
pub const ZERO: Self

The value of 0 represented by this unsigned integer type.

pub const ONE: Self

The value of 1 represented by this unsigned integer type.

pub const TWO: Self

The value of 2 represented by this unsigned integer type.

pub const THREE: Self

The value of 3 represented by this unsigned integer type.

pub const FOUR: Self

The value of 4 represented by this unsigned integer type.

pub const FIVE: Self

The value of 5 represented by this unsigned integer type.

pub const SIX: Self

The value of 6 represented by this unsigned integer type.

pub const SEVEN: Self

The value of 7 represented by this unsigned integer type.

pub const EIGHT: Self

The value of 8 represented by this unsigned integer type.

pub const NINE: Self

The value of 9 represented by this unsigned integer type.

pub const TEN: Self

The value of 10 represented by this unsigned integer type.